/**
 * Copyright (C) 1999, 2000, 2001  Free Software Foundation, Inc.
 *
 * This file is part of GNU gengetopt 
 *
 * GNU gengetopt is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2, or (at your option) 
 * any later version. 
 *
 * GNU gengetopt is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License along 
 * with gengetopt; see the file COPYING. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
 */


%{
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <time.h>

#include "argsdef.h"

#include "gengetopt.h"

   static int gengetopt_package_given = 0;
   static int gengetopt_version_given = 0;
   static int gengetopt_purpose_given = 0;

   extern void yyerror ( char *error ) ;
   extern int yylex () ;

   int gengetopt_add_option (char * long_opt, char short_opt, char * desc, 
                             int type, int flagstat, int required);


#define YYERROR_VERBOSE 1
#define check_result \
	if (o) { switch (o) { case 1: yyerror ("not enough memory"); break; case 2: yyerror ("long option redefined"); break; case 3: yyerror ("short option redefined"); break; case 4: yyerror ("bug found!!"); break; } YYERROR; }

   char packagename[90];
   char packageupper[90];
   char versionname[90];
   char purposetext[90];
   int headerprinted = 0;
   int gengetopt_count_line = 0;

   void upperstring(char *s) {
      int i;

      for (i=0; i < strlen(s); i++) {
         s[i] = toupper(s[i]);
      }
   }
 
   int gengetopt_define_package( char *s ) {
      gengetopt_package_given = 1;
      strcpy(packagename, s);
      strcpy(packageupper, s);
      upperstring(packageupper);
      return 0;
   }

   int gengetopt_define_version (char * s) {
      gengetopt_version_given = 1;
      strcpy(versionname, s);
      return 0;
   }
   int gengetopt_define_purpose (char * s) {
      gengetopt_purpose_given = 1;
      strcpy(purposetext, s);
      return 0;
   }

   int header(void) {
      char date[80];
      time_t t = time(NULL);
    
      strftime(date, sizeof(date)-1, "%B %Y", localtime(&t));
      headerprinted = 1;
      printf(".\\\" DO NOT MODIFY THIS FILE.  It was generated by ggo2man.\n"
             ".TH %s \"1\" \"%s\" \"%s %s\" \"User Commands\"\n"
             ".SH NAME\n"
             "%s - %c%s\n",
             packageupper, date, packagename, versionname,
             packagename, tolower(*purposetext), purposetext+1);
      printf(".SH SYNOPSIS\n"
             ".B %s [\\fIOPTIONS\\fR]\n"
             ".SH DESCRIPTION\n", packagename);
      gengetopt_add_option( "help", 'h', "Print help and exit",
                            ARG_NO, 0, 0 );
      gengetopt_add_option( "version", 'V', "Print version and exit",
                            ARG_NO, 0, 0 );

      return 0;
   }

   void removeescape( char *s ) {
      char *a;

      for (a = s; *a; a++ ) {
         if (*a == '\\') {
            switch( a[1] ) {
               case 't' : strncpy(a, " \t", 2); break;
               case 'n' : strncpy(a, " \n", 2); break;
               case 'r' : strncpy(a, " \n", 2); break;
               default: break;
            }
         }
      }
   }
 
   int gengetopt_add_option (char * long_opt, char short_opt, char * desc, 
                             int type, int flagstat, int required) {
      if( !headerprinted ) {
         header();
         headerprinted = 1;
      }
      printf(".TP\n");
      if( short_opt != '-' ) {
         printf("\\fB-%c\\fR, ", short_opt);
      }
      printf("\\fB--%s\\fR", long_opt);
      switch(type) {
         case ARG_NO: break;
         case ARG_FLAG: break;
         case ARG_STRING: printf("=\\fISTRING"); break;
         case ARG_INT: printf("=\\fIINT"); break;
         case ARG_SHORT: printf("=\\fISHORT"); break;
         case ARG_LONG: printf("=\\fILONG"); break;
         case ARG_FLOAT: printf("=\\fIFLOAT"); break;
         case ARG_DOUBLE: printf("=\\fIDOUBLE"); break;
         case ARG_LONGDOUBLE: printf("=\\fILONGDOUBLE"); break;
         case ARG_LONGLONG: printf("=\\fILONGLONG"); break;
         default: printf("=\\fIundefined"); break;
      }
      removeescape(desc);
      printf("\\fR\n%s\n", desc);
      return 0;
   }

   %}

%union {
char * str;
char chr;
int argtype;
int bool;
}

%token           TOK_PACKAGE
%token           TOK_VERSION
%token           TOK_OPTION
%token           TOK_YES
%token           TOK_NO
%token           TOK_FLAG
%token           TOK_PURPOSE
%token <bool>    TOK_ONOFF
%token <str>     TOK_STRING
%token <str>     TOK_MLSTRING
%token <chr>     TOK_CHAR
%token <argtype> TOK_ARGTYPE
%token <str>     TOK_LITERAL
%type  <str>     exp_str
%type  <str>     exp_mlstr
%type  <bool>    exp_yesno


%%


input:	/* empty */
	| input line
;


line:	'\n'
	| exp '\n'
;


exp_str:	 TOK_STRING             { $$ = $1; }
;

exp_mlstr:	 TOK_MLSTRING           { $$ = $1; }
;


exp_yesno:	  TOK_YES  { $$ = 1; }
		| TOK_NO   { $$ = 0; }
;


exp: TOK_PACKAGE TOK_STRING { if (gengetopt_package_given) { yyerror ("package redefined"); YYERROR; } else { gengetopt_package_given = 1; if (gengetopt_define_package ($2)) { yyerror ("not enough memory"); YYERROR; } } }
;


exp: TOK_VERSION TOK_STRING { if (gengetopt_version_given) { yyerror ("version redefined"); YYERROR; } else { gengetopt_version_given = 1; if (gengetopt_define_version ($2)) { yyerror ("not enough memory"); YYERROR; } } }
;

exp: TOK_PURPOSE exp_mlstr { if (gengetopt_purpose_given) { yyerror ("purpose redefined"); YYERROR; } else { gengetopt_purpose_given = 1; if (gengetopt_define_purpose ($2)) { yyerror ("not enough memory"); YYERROR; } } }
;

exp: TOK_PURPOSE exp_str { if (gengetopt_purpose_given) { yyerror ("purpose redefined"); YYERROR; } else { gengetopt_purpose_given = 1; if (gengetopt_define_purpose ($2)) { yyerror ("not enough memory"); YYERROR; } } }
;
exp: TOK_OPTION TOK_STRING TOK_CHAR exp_str TOK_NO { int o = gengetopt_add_option ($2, $3, $4, ARG_NO, 0, 0); check_result; }
;


exp: TOK_OPTION TOK_STRING TOK_CHAR exp_str TOK_FLAG TOK_ONOFF { int o = gengetopt_add_option ($2, $3, $4, ARG_FLAG, $6, 0); check_result; }
;

     
exp: TOK_OPTION TOK_STRING TOK_CHAR exp_str TOK_ARGTYPE exp_yesno { int o = gengetopt_add_option ($2, $3, $4, $5, 0, $6); check_result; }
;

exp: TOK_LITERAL { if(!headerprinted) header(); printf("%s\n", $1); }
;


%%

void yyerror(char *s) {
   fprintf(stderr, "%s\n", s);
}

void print_compilation_data( void ) {
#ifdef _MSC_VER
   printf("\n\tCompiled with Microsoft Visual C++: %d, ", _MSC_VER);
#elif defined __WATCOMC__
   printf("\n\tCompiled with Watcom version %.2f, ", __WATCOMC__);
#elif defined __GNUC__
   printf("\n\tCompiled with GCC version %d.%d, ", __GNUC__, __GNUC_MINOR__);
#endif
   printf(__DATE__ ", " __TIME__ ".\n");
}

#include "ggo2mancmdline.h"

int main(int argc, char **argv) {
   struct gengetopt_args_info ai;

   cmdline_parser(argc, argv, &ai);
   if (ai.version_given) {
      cmdline_parser_print_version();
      print_compilation_data();
      return 0;
   }
   
   yyparse();
   if(!headerprinted) header();
   return 0;
}
